﻿using Hotline.Orders;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.Order;
using Hotline.Share.Tools;
using Hotline.Snapshot;
using Hotline.Snapshot.Interfaces;
using Mapster;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Application.Snapshot;
public class RedPackApplication : IRedPackApplication, IScopeDependency
{
    private readonly IOrderSnapshotRepository _orderSnapshotRepository;
    private readonly IOrderRepository _orderRepository;
    private readonly ISnapshotSMSTemplateRepository _snapshotSMSTemplateRepository;
    private readonly IIndustryRepository _industryRepository;
    private readonly IRedPackAuditRepository _redPackAuditRepository;

    public RedPackApplication(IOrderSnapshotRepository orderSnapshotRepository, ISnapshotSMSTemplateRepository snapshotSMSTemplateRepository, IOrderRepository orderRepository, IIndustryRepository industryRepository, IRedPackAuditRepository redPackAuditRepository)
    {
        _orderSnapshotRepository = orderSnapshotRepository;
        _snapshotSMSTemplateRepository = snapshotSMSTemplateRepository;
        _orderRepository = orderRepository;
        _industryRepository = industryRepository;
        _redPackAuditRepository = redPackAuditRepository;
    }

    /// <summary>
    /// 获取审核详情
    /// </summary>
    /// <param name="id">工单Id</param>
    /// <returns></returns>
    public async Task<SnapshotOrderAuditDetailOutDto> GetRedPackAuditDetailAsync(string id)
    {
        var order = await _orderRepository.GetAsync(id) ?? throw UserFriendlyException.SameMessage("工单不存在");
        var outDto = new SnapshotOrderAuditDetailOutDto { Order = order.Adapt<SnapshotOrderAuditOrderDetailOutDto>() };
        var industry = await _industryRepository.Queryable(includeDeleted: true)
            .LeftJoin<OrderSnapshot>((i, o) => i.Id == o.IndustryId)
            .Select((i, o) => new { i.Id, i.CitizenReadPackAmount })
            .FirstAsync();
        outDto.Amount = industry.CitizenReadPackAmount;
        return outDto;
    }

    public ISugarQueryable<SnapshotOrderAuditItemsOutDto> GetRedPackAuditItemsAsync(SnapshotOrderAuditItemsInDto dto)
    {
        var query = _redPackAuditRepository.Queryable(includeDeleted: true)
            .LeftJoin<Order>((redPackAudit, order) => redPackAudit.OrderId == order.Id)
            .LeftJoin<OrderSnapshot>((redPackAudit, order, snapshot) => redPackAudit.OrderId == snapshot.Id)
            .LeftJoin<RedPackRecord>((redPackAudit, order, snapshot, record) => redPackAudit.Id == record.OrderId)
            .LeftJoin<Industry>((redPackAudit, order, snapshot, record, industry) => snapshot.IndustryId == industry.Id)
            .Where((redPackAudit, order, snapshot, record, industry) => order.Status == EOrderStatus.Filed)
            .WhereIF(dto.No.NotNullOrEmpty(), (redPackAudit, order, snapshot, record, industry) => order.No.Contains(dto.No))
            .WhereIF(dto.Title.NotNullOrEmpty(), (redPackAudit, order, snapshot, record, industry) => order.Title.Contains(dto.Title))
            .WhereIF(dto.FromPhone.NotNullOrEmpty(), (redPackAudit, order, snapshot, record, industry) => order.FromPhone.Contains(dto.FromPhone))
            .WhereIF(dto.BeginCreationTime.HasValue && dto.EndCreationTime.HasValue, (redPackAudit, order, snapshot, record, industry) => order.CreationTime <= dto.EndCreationTime && order.CreationTime >= dto.BeginCreationTime)
            .WhereIF(dto.IsDeal.HasValue, (redPackAudit, order, snapshot, record, industry) => snapshot.IsDeal == dto.IsDeal)
            .WhereIF(dto.IsTruth.HasValue, (redPackAudit, order, snapshot, record, industry) => snapshot.IsTruth == dto.IsTruth)
            .WhereIF(dto.BeginAuditTime.HasValue && dto.EndAuditTime.HasValue, (redPackAudit, order, snapshot, record, industry) => redPackAudit.AuditTime <= dto.EndAuditTime && redPackAudit.AuditTime >= dto.BeginAuditTime)
            .WhereIF(dto.IsIssued.HasValue, (redPackAudit, order, snapshot, record, industry) => redPackAudit.IsIssued == dto.IsIssued)
            .WhereIF(dto.IndustryId.NotNullOrEmpty(), (redPackAudit, order, snapshot, record, industry) => snapshot.IndustryId == dto.IndustryId)
            .WhereIF(dto.ConfigAmount.HasValue, (redPackAudit, order, snapshot, record, industry) => industry.CitizenReadPackAmount == dto.ConfigAmount)
            .WhereIF(dto.AcutalAmount.HasValue, (redPackAudit, order, snapshot, record, industry) => redPackAudit.AcutalAmount == dto.AcutalAmount)
            .WhereIF(dto.ApprovedAmount.HasValue, (redPackAudit, order, snapshot, record, industry) => redPackAudit.ApprovedAmount == dto.ApprovedAmount)
            .WhereIF(dto.IsDanger.HasValue, (redPackAudit, order, snapshot, record, industry) => snapshot.IsDanger == dto.IsDanger)
            .WhereIF(dto.Status.HasValue, (redPackAudit, order, snapshot, record, industry) => redPackAudit.Status == dto.Status)
            .Select((redPackAudit, order, snapshot, record, industry) => new SnapshotOrderAuditItemsOutDto
            {
                Id = redPackAudit.Id,
                RedPackAuditId = redPackAudit.Id,
                OrderId = order.Id,
                No = order.No,
                Title = order.Title,
                IndustryName = industry.Name,
                IndustryId = industry.Id,
                SourceChannel = order.SourceChannel,
                SourceChannelCode = order.SourceChannelCode,
                Status = order.Status,
                IsDanger = snapshot.IsDanger,
                FromPhone = order.FromPhone,
                FromName = order.FromName,
                AuditTime = redPackAudit.AuditTime,
                ApprovedAmount = redPackAudit.ApprovedAmount,
                AcutalAmount = redPackAudit.AcutalAmount,
                IsIssued = redPackAudit.IsIssued,
                RecordRemark = record.Remark,
                County = order.County,
                // IsRectify = s.IsRepetition
                IsDeal = snapshot.IsDeal,
                NetworkENumber = snapshot.NetworkENumber,
                IsTruth = snapshot.IsTruth,
                IsRepetition = snapshot.IsRepetition,
                CreationTime = order.CreationTime,
                OrgLevelOneCode = order.OrgLevelOneCode,
                OrgLevelOneName = order.OrgLevelOneName,
                AuditId = redPackAudit.AuditId,
                AuditName = redPackAudit.AuditName,
                AuditOrgId = redPackAudit.AuditOrgId,
                AuditOrgName = redPackAudit.AuditOrgName,
                AuditRemark = redPackAudit.AuditRemark,
                BankCardNo = record.BankCardNo,
                OpenBank = record.OpenBank,
            });
        return query;
    }

    /// <summary>
    /// 获取审核短信模板
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public async Task<IList<GetRedPackAuditSMSTemplateOutDto>> GetRedPackAuditSMSTemplateAsync(GetRedPackAuditSMSTemplateInDto dto)
    {
        var items = await _snapshotSMSTemplateRepository.Queryable(includeDeleted: true)
            .LeftJoin<Industry>((sms, industry) => sms.IndustryId == industry.Id)
            .LeftJoin<OrderSnapshot>((sms, industry, snapshot) => snapshot.IndustryId == industry.Id)
            .Where((sms, industry, snapshot) => (sms.IsPublic == true || sms.IndustryId == snapshot.IndustryId) 
             && sms.Status == dto.Status && sms.IsEnable == true)
            .Select<GetRedPackAuditSMSTemplateOutDto>()
            .Distinct()
            .ToListAsync();
        return items;
    }
}
